@@ -15,7 +15,7 @@ def test_job():
         assert len(ds) == 2
 
         print("setting up alignment job...")
-        j = job.Job(cfg, ds, tmp, batch_size=1, n_workers=1)
+        j = job.Job(cfg, ds, tmp, batch_size=1, n_workers=1, gpu=False)
 
         # construct the generic model text cleaner
         ds.clean_text_fn = text.TextCleaner(cfg.language, j.aligner.vocab())
@@ -24,11 +24,11 @@ def test_job():
         j.run()
 
         one = utils.read_alignment(tmp, "audio/one.mp3")
-        assert len(one.word_segments) == 1
-        assert one.word_segments[0].label == "one"
-        assert one.word_segments[0].score > 0.9
+        assert len(one.words_cleaned) == 1
+        assert one.words_cleaned[0].label == "one"
+        assert one.words_cleaned[0].score > 0.9
 
         two = utils.read_alignment(tmp, "audio/two.mp3")
-        assert len(two.word_segments) == 1
-        assert two.word_segments[0].label == "two"
-        assert two.word_segments[0].score > 0.9
+        assert len(two.words_cleaned) == 1
+        assert two.words_cleaned[0].label == "two"
+        assert two.words_cleaned[0].score > 0.9
